'****************************************************************************
' Debugger.vbs v20030208
' Copyright (c) 2001-2003 Kenneth Gober
' All Rights Reserved
' This software is freely redistributable, with the following restrictions:
'   1. The copyright notice above, and these restrictions, may not be removed.
'   2. This software may not be sold in whole or in part, by itself or in
'      combination with other software, without prior written permission.
'   3. This software may not be included in any collection or archive if such
'      collection is sold for profit, without prior written permission.
'   4. "Prior written permission" may only be granted by the author or his
'      agents.  No waiver of these requirements is granted if you are not
'      able to locate the author or his agents.
'   5. These restrictions do not apply to the authors of SkunkWorks.
'****************************************************************************

'****************************************************************************
' Version History:
' 20021229 - initial version
' 20030205 - move start/stop code to StartHandler() and StopHandler()
'            add '/sw eval <expr>' and '/sw exec <stmt>' commands
'            show byte/int/long results in decimal and hex
'            show more descriptive error information
' 20030208 - add '/sw debugger echo' commands
'
' To do:
'****************************************************************************

Option Explicit


'****************************************************************************
' Debugger - evaluates in-game commands
'
' issue the command "debugger start" to enable
' issue the command "debugger stop" to disable
' when enabled, issue debugger commands as follows:
'   {eval <expression>} - evaluate expression and display result
'   {exec <statement>} - execute statement
'   eval <expression> - evaluate expression and display result
'   exec <statement> - execute statement
'****************************************************************************

Dim Debugger_Monitor, Debugger_Handler, Debugger_Running
Dim Debugger_MonitorMode, Debugger_MonitorColor
Dim Debugger_OutputMode, Debugger_OutputColor

Class DebuggerMonitorClass
  Public Sub OnCommand(szCmd)
    If (StrComp(szCmd, "debugger start", 1) = 0) Then
      If Not (Debugger_Running = True) Then
        Call StartHandler()
        Call skapi.OutputLine("Debugger Started", Debugger_MonitorMode, Debugger_MonitorColor)
      End If
    ElseIf (StrComp(szCmd, "debugger stop", 1) = 0) Then
      If (Debugger_Running = True) Then
        Call StopHandler()
        Call skapi.OutputLine("Debugger Stopped", Debugger_MonitorMode, Debugger_MonitorColor)
      End If
    End If
  End Sub

  Public Sub OnLogon(acoChar)
    If IsEmpty(Debugger_Running) Then
      Call StartHandler()
      Call skapi.OutputLine("Debugger Started", Debugger_MonitorMode, Debugger_MonitorColor)
    ElseIf (Debugger_Running = True) Then
      Call skapi.OutputLine("Debugger Running", Debugger_MonitorMode, Debugger_MonitorColor)
    Else
      Call skapi.OutputLine("Debugger Disabled", Debugger_MonitorMode, Debugger_MonitorColor)
    End If
  End Sub

  Private Sub StartHandler()
    Set Debugger_Handler = New DebuggerHandlerClass
    Call skapi.AddHandler(63, Debugger_Handler)
    Debugger_Running = True
  End Sub

  Private Sub StopHandler()
    Debugger_Running = False
    Call skapi.RemoveHandler(63, Debugger_Handler)
    Set Debugger_Handler = Nothing
  End Sub
End Class

Class DebuggerHandlerClass
  Private EchoState

  Private Sub Class_Initialize()
    EchoState = False
  End Sub

  Public Sub OnCommand(szCmd)
    Dim buf
    If (Left(szCmd, 1) = "{") And (Right(szCmd, 1) = "}") Then
      If (StrComp(Left(szCmd, 6), "{eval ", 1) = 0) Then
        Call EvalExpression(Mid(szCmd, 7, Len(szCmd) - 7))
      ElseIf (StrComp(Left(szCmd, 6), "{exec ", 1) = 0) Then
        Call ExecStatement(Mid(szCmd, 7, Len(szCmd) - 7))
      End If
    ElseIf (StrComp(Left(szCmd, 5), "eval ", 1) = 0) Then
      Call EvalExpression(Mid(szCmd, 6))
    ElseIf (StrComp(Left(szCmd, 5), "exec ", 1) = 0) Then
      Call ExecStatement(Mid(szCmd, 6))
    ElseIf (StrComp(szCmd, "debugger echo", 1) = 0) Then
      buf = "Debugger: Echo is "
      If (EchoState = True) Then
        buf = buf & "On"
      Else
        buf = buf & "Off"
      End If
      Call skapi.OutputLine(buf, Debugger_OutputMode, Debugger_OutputColor)
    ElseIf (StrComp(szCmd, "debugger echo on", 1) = 0) Then
      EchoState = True
      Call skapi.OutputLine("Debugger: Echo is On", Debugger_OutputMode, Debugger_OutputColor)
    ElseIf (StrComp(szCmd, "debugger echo off", 1) = 0) Then
      EchoState = False
      Call skapi.OutputLine("Debugger: Echo is Off", Debugger_OutputMode, Debugger_OutputColor)
    End If
  End Sub

  Private Sub EvalExpression(expr)
    Dim r
    On Error Resume Next
    If (EchoState = True) Then Call skapi.OutputLine("Debugger: Eval " & expr, Debugger_OutputMode, Debugger_OutputColor)
    r = Eval(expr)
    If (Err.Number <> 0) Then
      Call skapi.OutputLine("Debugger: Error " & Err.Number & " (" & HexStr(Err.Number, 8) & ") " & Err.Description, Debugger_OutputMode, Debugger_OutputColor)
    Else
      Select Case VarType(r)
        Case vbByte
          Call skapi.OutputLine("Debugger: Result " & r & " (" & HexStr(r, 2) & ")", Debugger_OutputMode, Debugger_OutputColor)
        Case vbInteger
          Call skapi.OutputLine("Debugger: Result " & r & " (" & HexStr(r, 4) & ")", Debugger_OutputMode, Debugger_OutputColor)
        Case vbLong
          Call skapi.OutputLine("Debugger: Result " & r & " (" & HexStr(r, 8) & ")", Debugger_OutputMode, Debugger_OutputColor)
        Case Else
          Call skapi.OutputLine("Debugger: Result " & r, Debugger_OutputMode, Debugger_OutputColor)
      End Select
    End If
  End Sub

  Private Sub ExecStatement(stmt)
    On Error Resume Next
    If (EchoState = True) Then Call skapi.OutputLine("Debugger: Exec " & stmt, Debugger_OutputMode, Debugger_OutputColor)
    Execute stmt
    If (Err.Number <> 0) Then
      Call skapi.OutputLine("Debugger: Error " & Err.Number & " (" & HexStr(Err.Number, 8) & ") " & Err.Description, Debugger_OutputMode, Debugger_OutputColor)
    End If
  End Sub

  Private Function HexStr(value, places)
    HexStr = Right(String(places, "0") & Hex(value), places)
  End Function
End Class

Debugger_MonitorMode = 16
Debugger_MonitorColor = 13
Debugger_OutputMode = 16
Debugger_OutputColor = 13
Set Debugger_Monitor = New DebuggerMonitorClass
Call skapi.AddHandler(32, Debugger_Monitor)
Call skapi.AddHandler(63, Debugger_Monitor)
